<?php

namespace v4\wp\plugins\oauth;

use Exception;
use v4Webhook;

class Setup
{
    // singleton class pattern:
    protected static ?Setup $instance = null;

    public static function getInstance(): Setup
    {
        if (is_null(self::$instance)) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    public function __construct()
    {
        if (!defined('ABSPATH')) {
            die;
        }

        require_once __DIR__ . '/vendor/autoload.php';
        require_once __DIR__ . '/Identity.php';
        require_once __DIR__ . '/Settings.php';
        require_once __DIR__ . '/User.php';

        //settings
        new Settings();

        add_action('init', [$this, 'init']);
        add_action('plugins_loaded', [$this, 'localization']);
        add_filter('woocommerceEmailActions', [$this, 'woocommerceEmailActions']);
        add_action('admin_init', [$this, 'restrictBackend']);

        // check email in 1crm
        add_filter('email_exists', [$this, 'emailExists1crm'], 20, 2);
    }

    public function init()
    {
        if ((is_user_logged_in() || isset($_GET['v4oauth-result']))
            && session_status() == PHP_SESSION_NONE
        ) {
            session_start();
        }

        // hook the query_vars and template_redirect, so we can stay within the WordPress context no matter what
        // (avoids having to use wp-load.php)
        add_action('template_redirect', array($this, 'qVarHandlers'));

        add_action('wp_logout', array($this, 'logout'));

        // push login messages into the DOM if the setting is enabled:
        add_action('wp_footer', array($this, 'pushLoginMessages'));
        add_filter('admin_footer', array($this, 'pushLoginMessages'));
        add_filter('login_footer', array($this, 'pushLoginMessages'));

        // WooCommerce API
        // TODO check what that line was supposed to do. Function doesn't exist
        //  add_action('woocommerce_webhook_delivery', 'logout_on_delivery_failure', 10, 2);

        //scripts
        wp_enqueue_style('v4oa', plugin_dir_url(__FILE__) . 'assets/css/v4oauth.css');
        wp_enqueue_script('v4oa', plugin_dir_url(__FILE__) . 'assets/js/v4oauth.js', array(), false, true);
    }

    public function emailExists1crm($user_id, $email)
    {
        global $v4oauth_1crm_emails;
        if (!$v4oauth_1crm_emails) {
            $v4oauth_1crm_emails = [];
        }

        if (!$user_id) {
            // don't ask 1crm if we already know the email exists
            if (in_array($email, $v4oauth_1crm_emails)) {
                return -1;
            }

            $data = [
                'case' => 'check_email',
                'email' => $email,
                'billing' => [
                    'last_name' => 'empty, but irrelevant on check_email'
                ]
            ];
            $json = json_encode($data);
            try {
                require_once WP_PLUGIN_DIR . '/v4-wp-webhook/includes/v4Webhook.php';
                v4Webhook::fire_webhook($json, 'customer');
            } catch (Exception $e) {
                $msg = $e->getMessage();
                if ($msg == "E-Mail already exists") {
                    $v4oauth_1crm_emails[] = $email;
                } else {
                    error_log('v4oAuth error (' . $e->getCode() . ') ' . $msg);
                }
                // either exists, or error. tell function that there is (might be) that email
                return -1;
            }
        }

        return $user_id;
    }

    public function restrictBackend()
    {
        if (!current_user_can('edit_posts') &&
            ('/wp-admin/admin-ajax.php' != $_SERVER['PHP_SELF']) &&
            ('/wp-admin/admin-post.php' != $_SERVER['PHP_SELF'])
        ) {
            wp_redirect(home_url());
            exit;
        }
    }

    public function woocommerceEmailActions($email_actions)
    {
        foreach ($email_actions as $k => $email_action) {
            if ($email_action == 'woocommerce_created_customer') {
                unset($email_actions[$k]);
            }
        }
        return $email_actions;
    }

    public function localization()
    {
        // LOCALIZATION
        load_plugin_textdomain(
            'v4-oauth',
            false,
            untrailingslashit(dirname(plugin_basename(__FILE__))) . '/languages'
        );
    }

    // define the querystring variables that should trigger an action:
    public function qVarTriggers(): array
    {
        return [
            'connect',
            'code',
            'state',
            'lost_password',
            'change_password',
        ];
    }

    // handle the querystring triggers:
    public function qVarHandlers()
    {
//        //on WooCommerce Checkout
//        if (function_exists('is_checkout') && is_checkout()) {
//            //TODO update user?
//        }

        //on query var trigger
        foreach ($this->qVarTriggers() as $qVar_trigger) {
            if (isset($_GET[$qVar_trigger])) {
                if (session_status() == PHP_SESSION_NONE) {
                    session_start();
                }

                do_action('before_oauth');

                try {
                    $Identity = Identity::getInstance();

                    switch ($qVar_trigger) {
                        case 'lost_password':
                            $Identity->lostPassword();

                            break;
                        case 'change_password':
                            $Identity->changePassword();

                            break;
                        default:
                            if ($Identity->auth()) {
                                $User = new User($Identity);
                                $User->login();
                            }

                            break;
                    }
                } catch (Exception $e) {
                    $_SESSION['V4OA']['RESULT'] = __($e->getMessage(), 'v4-oauth');
                    $_SESSION['V4OA']['ERROR'] = true;
                }

                do_action('after_oauth');

                $delimiter = strpos($_SESSION['V4OA']['LAST_URL'], '?') ? '&' : '?';
                $url = $_SESSION['V4OA']['LAST_URL'] . $delimiter . 'v4oauth-result';

                header("Location: " . $url);
                exit;
            }
        }
    }

    public function logout()
    {
        $Identity = Identity::getInstance();
        $Identity->logout();
    }

    public function pushLoginMessages()
    {
        if (!empty($_SESSION['V4OA']['RESULT'])) {
            $result = $_SESSION['V4OA']['RESULT'];
            unset($_SESSION['V4OA']['RESULT']);

            //is error?
            $css_class = '';
            if (isset($_SESSION['V4OA']['ERROR'])) {
                $css_class = 'error';
                unset($_SESSION['V4OA']['ERROR']);
            }

            echo "<div id='v4oauth-result' class='$css_class'>$result<span id='v4oauth-result-close'></span></div>";
        }
    }
}
